@@ -100,6 +100,10 @@ class latex2edx(object):
                  do_images=True,
                  update_policy=False,
                  suppress_policy=False,
+                 suppress_verticals=False,
+                 section_only=False,
+                 xml_only=False,
+                 units_only=False,
                  ):
         '''
         extra_xml_filters = list of functions acting on XML, applied to XHTML
@@ -130,6 +134,11 @@ class latex2edx(object):
         self.do_merge = do_merge
         self.update_policy = update_policy
         self.suppress_policy = suppress_policy
+        self.section_only = section_only
+        self.suppress_verticals = suppress_verticals
+        self.xml_only = xml_only
+        self.units_only = units_only
+        self.the_xml = None
 
         if output_fn is None or not output_fn:
             if fn.endswith('.tex'):
@@ -160,17 +169,98 @@ class latex2edx(object):
 
         self.URLNAMES = []
 
+
+    def save_xml(self):
+        '''
+        Save XML file (as .xbundle, normally) to the output_fn
+        '''
+        open(self.output_fn,'w').write(etree.tostring(self.xml, pretty_print=True))
+
+    def export_sections_only(self):
+        '''
+        Export sequentials only (no course, no chapters).
+        Also save the initial XML as the xbundle file
+        '''
+        self.save_xml()
+        xb = xbundle.XBundle(force_studio_format=(not self.suppress_verticals), keep_urls=True)
+        xb.dir = self.output_dir
+
+        tags = ['sequential', 'problem', 'html', 'video']
+        for tag in tags:
+            print "    %s: %d" % (tag, len(self.xml.findall('.//%s' % tag)))
+
+        for seq in self.xml.findall('.//sequential'):
+            nprob = len(seq.findall('.//problem'))
+            nhtml = len(seq.findall('.//html'))
+            print "--> exporting sequential %s (%d problem, %d html)" % (seq.get('display_name', '<unknown display_name>'),
+                                                                         nprob, nhtml)
+            xb.add_descriptors(seq)
+            xb.export_xml_to_directory(seq, dowrite=True)
+
+    def export_units_only(self):
+        '''
+        Export units (problem, html, video) only (no course, no chapters).
+        Also save the initial XML as the xbundle file
+        '''
+        self.save_xml()
+        xb = xbundle.XBundle(force_studio_format=(not self.suppress_verticals), keep_urls=True)
+        xb.dir = self.output_dir
+
+        tags = ['problem', 'html', 'video']
+        for tag in tags:
+            print "    %s: %d" % (tag, len(self.xml.findall('.//%s' % tag)))
+
+        for tag in tags:
+            for unit in self.xml.findall('.//%s' % tag):
+                print "--> exporting %s (%s) url_name=%s" % (unit.get('display_name', '<unknown display_name>'), 
+                                                             self.get_filename_and_linenum(unit),
+                                                             unit.get('url_name', '<unknown>'),
+                                                         )
+                xb.add_descriptors(unit)
+                xb.export_xml_to_directory(unit, dowrite=True)
+
+
+    @property
+    def xml(self):
+        '''
+        Compute our XML representation by parsing the XHTML from plastex into XML, then running it through
+        all the fix_filters.
+
+        Cache result, so we only do the computation once.
+        '''
+        if self.the_xml is None:
+            xml = etree.fromstring(self.xhtml)
+            for filter in self.fix_filters:
+                filter(xml)
+            self.the_xml = xml
+        return self.the_xml
+
+
     def convert(self):
         '''
         Convert xhtml to xbundle and then xbundle to directory of XML files.
         if self.do_merge then do not overwrite course files; attempt to merge them.
         '''
+        if self.section_only and self.xml_only:
+            print "Saving XML to file %s" % self.output_fn
+            return self.save_xml()
+
+        if self.section_only:
+            # if section_only then only export edXsections (sequentials)
+            return self.export_sections_only()
+
+        if self.units_only:
+            return self.export_units_only()
+
         self.xhtml2xbundle()
         self.xb.save(self.output_fn)
         print "xbundle generated (%s): " % self.output_fn
-        tags = ['chapter', 'sequential', 'problem', 'html']
+        tags = ['chapter', 'sequential', 'problem', 'html', 'video']
         for tag in tags:
             print "    %s: %d" % (tag, len(self.xb.course.findall('.//%s' % tag)))
+        if self.xml_only:
+            print "Saved xbundle XML to file %s" % self.output_fn
+            return
         self.xb.export_to_directory(self.output_dir, xml_only=True)
         print "Course exported to %s/" % self.output_dir
 
@@ -203,12 +293,9 @@ class latex2edx(object):
         Convert XHTML output of PlasTeX to an edX xbundle file.
         Use lxml to parse the XML and extract the desired parts.
         '''
-        xml = etree.fromstring(self.xhtml)
-        self.xml = xml
-        for filter in self.fix_filters:
-            filter(xml)
+        xml = self.xml
         no_overwrite = ['course'] if self.do_merge else []
-        xb = xbundle.XBundle(force_studio_format=True, keep_urls=True,
+        xb = xbundle.XBundle(force_studio_format=(not self.suppress_verticals), keep_urls=True,
                              no_overwrite=no_overwrite)
         xb.KeepTogetherTags = ['sequential', 'vertical', 'conditional']
         course = xml.find('.//course')
@@ -235,10 +322,10 @@ class latex2edx(object):
         if (not self.suppress_policy) and (not self.update_policy):
             return
 
-        def fixdate(dt):
-            dt = date_parse(dt)
+        def fixdate(dtin):
+            dt = date_parse(dtin)
             if dt is None:
-                print "--> Error: bad date %s given for policy setting" % dt
+                print "--> Error: bad date '%s' given for policy setting" % dtin
                 raise
             return dt.strftime('%Y-%m-%dT%H:%M')
 
@@ -281,7 +368,11 @@ class latex2edx(object):
                         if ffun is None:
                             sval = val
                         else:
-                            sval = ffun(val)
+                            try:
+                                sval = ffun(val)
+                            except Exception as err:
+                                msg = "Error processing element %s in %s" % (elem.tag, self.get_filename_and_linenum(elem))
+                                raise Exception(msg)
                         policy[key][setting] = sval
 
             copy_settings(course, policy)		    # do course first
@@ -552,8 +643,7 @@ class latex2edx(object):
             showhide.attrib.pop('id')
             showhide.attrib.pop('description')
     
-    @staticmethod
-    def process_include(tree, do_python=False):
+    def process_include(self, tree, do_python=False):
         '''
         Include XML or python file.
 
@@ -566,21 +656,20 @@ class latex2edx(object):
             cmd += "py"
         for include in tree.findall(tag):
             incfn = include.text
+            linenum = include.get('linenum','<unavailable>')
+            texfn = include.get('filename','<unavailable>')
             if incfn is None:
                 print "Error: %s must specify file to include!" % cmd
-                print "See xhtml source line %s" % getattr(include,'linenum','<unavailable>')
-                raise
+                raise Exception(self.standard_error_msg(include))
             incfn = incfn.strip()
             if not os.path.exists(incfn):
                 print "Error: include file %s does not exist!" % incfn
-                print "See xhtml source line %s" % getattr(include,'linenum','<unavailable>')
-                raise
+                raise Exception(self.standard_error_msg(include))
             try:
                 incdata = open(incfn).read()
             except Exception, err:
                 print "Error %s: cannot open include file %s to read" % (err,incfn)
-                print "See xhtml source line %s" % getattr(include,'linenum','<unavailable>')
-                raise
+                raise Exception(self.standard_error_msg(include))
 
             # if python script, then check its syntax
             if do_python:
@@ -589,7 +678,7 @@ class latex2edx(object):
                 except Exception as err:
                     print "Error in python script %s! Err=%s" % (incfn, err)
                     print "Aborting!"
-                    sys.exit(0)
+                    raise Exception(self.standard_error_msg(include))
 
             try:
                 if do_python:
@@ -598,17 +687,25 @@ class latex2edx(object):
                     incxml = etree.fromstring(incdata)
             except Exception, err:
                 print "Error %s parsing XML for include file %s" % (err,incfn)
-                print "See xhtml source line %s" % getattr(include,'sourceline','<unavailable>')
-                raise
+                print "See tex file %s line %s" % (texfn, linenum)
+                raise Exception(self.standard_error_msg(include))
     
-            print "--> including file %s at line %s" % (incfn,getattr(include,'sourceline','<unavailable>'))
+	    # remove parent <p> if it exists
+            parent = include.getparent()
+            pp = parent.getparent()
+            if parent.tag=='p' and not parent.text.strip() and pp is not None:
+                parent.addprevious(include)
+                pp.remove(parent)
+
+            print "--> including file %s at line %s" % (incfn, linenum)
             if incxml.tag=='html' and len(incxml)>0:		# strip out outer <html> container
                 for k in incxml:
                     include.addprevious(k)	
             else:
                 include.addprevious(incxml)
             p = include.getparent()
-            p.remove(include)
+            if p is not None:
+                p.remove(include)
 
     def process_includepy(self, tree):
         '''
@@ -616,6 +713,16 @@ class latex2edx(object):
         '''
         self.process_include(tree, do_python=True)
 
+    @staticmethod
+    def get_filename_and_linenum(elem):
+        linenum = elem.get('linenum','<unavailable>')
+        texfn = elem.get('tex_filename', elem.get('filename', '<unavailable>'))
+        return "file %s line %s" % (texfn, linenum)
+
+    def standard_error_msg(self, elem):
+        msg = "Error processing element %s in %s" % (elem.tag, self.get_filename_and_linenum(elem))
+        return msg
+
     def process_dndtex(self, tree):
         '''
         Handle \edXdndtex{dnd_file.tex} inclusion of latex2dnd tex inputs.
@@ -623,24 +730,26 @@ class latex2edx(object):
         tag = './/edxdndtex'
         for dndxml in tree.findall(tag):
             dndfn = dndxml.text
+            linenum = dndxml.get('linenum','<unavailable>')
+            texfn = dndxml.get('filename','<unavailable>')
             if dndfn is None:
                 print "Error: %s must specify dnd tex filename!" % cmd
-                print "See xhtml source line %s" % getattr(dndxml,'linenum','<unavailable>')
+                print "See tex file %s line %s" % (texfn, linenum)
                 raise
             dndfn = dndfn.strip()
             if not dndfn.endswith('.tex'):
                 print "Error: dnd file %s should be a .tex file!" % dndfn
-                print "See xhtml source line %s" % getattr(dndxml,'linenum','<unavailable>')
+                print "See tex file %s line %s" % (texfn, linenum)
                 raise
             if not os.path.exists(dndfn):
                 print "Error: dnd tex file %s does not exist!" % dndfn
-                print "See xhtml source line %s" % getattr(dndxml,'linenum','<unavailable>')
+                print "See tex file %s line %s" % (texfn, linenum)
                 raise
             try:
                 dndsrc = open(dndfn).read()
             except Exception, err:
                 print "Error %s: cannot open dnd tex file %s to read" % (err,dndfn)
-                print "See xhtml source line %s" % getattr(dndxml,'linenum','<unavailable>')
+                print "See tex file %s line %s" % (texfn, linenum)
                 raise
 
             # Use latex2dnd to compile dnd tex into edX XML.
@@ -723,14 +832,25 @@ class latex2edx(object):
         '''
         for script in tree.findall('.//script[@type="text/python"]'):
             pyfile = tempfile.NamedTemporaryFile(mode='w', delete=False)
-            pyfile.write(script.text)
+            if script.text is None:
+                print "Warning: empty script!"
+                print "Script location: %s" % etree.tostring(script)
+                continue
+            try:
+                pyfile.write(script.text)
+            except Exception as err:
+                print "Error checking python script %s" % script.text
+                print str(err)
+                print "Script location: %s" % etree.tostring(script)
+                continue
             pyfile.close()
             try:
                 py_compile.compile(pyfile.name, doraise=True)
             except Exception as err:
                 print "Error in python script %s! Err=%s" % (pyfile.name, err)
+                print "Script location: %s" % etree.tostring(script)
                 print "Aborting!"
-                sys.exit(0)
+                raise Exception(self.standard_error_msg(script))
             os.unlink(pyfile.name)
 
     def add_url_names(self, xml):
@@ -865,11 +985,32 @@ def CommandLine():
                       dest="suppress_policy",
                       default=False,
                       help="suppress policy settings from XML files",)
+    parser.add_option("--suppress-verticals",
+                      action="store_true",
+                      dest="suppress_verticals",
+                      default=False,
+                      help="do not automatically add extra verticals needed for Studio compatibility",)
+    parser.add_option("-S", "--section-only",
+                      action="store_true",
+                      dest="section_only",
+                      default=False,
+                      help="export only edXsections (sequentials) -- no course or chapters",)
+    parser.add_option("-x", "--xml-only",
+                      action="store_true",
+                      dest="xml_only",
+                      default=False,
+                      help="export only xbundle xml file -- no separate course content",)
+    parser.add_option("--units-only",
+                      action="store_true",
+                      dest="units_only",
+                      default=False,
+                      help="export only units, including problem, html -- no course, chapter, section",)
     (opts, args) = parser.parse_args()
 
     if len(args)<1:
-        parser.error('wrong number of arguments')
-        sys.exit(0)
+        print 'latex2edx: wrong number of arguments'
+        parser.print_help()
+        sys.exit(-2)
     fn = args[0]
 
     config = DEFAULT_CONFIG
@@ -883,6 +1024,10 @@ def CommandLine():
                   do_merge=opts.merge,
                   update_policy=opts.update_policy,
                   suppress_policy=opts.suppress_policy,
+                  suppress_verticals=opts.suppress_verticals,
+                  section_only=opts.section_only,
+                  xml_only=opts.xml_only,
+                  units_only=opts.units_only,
         )
     c.convert()
     
