@@ -6,13 +6,31 @@ from plasTeX import Base
 log = getLogger()
 status = getLogger('status')
 
+class MyBaseCommand(Base.Command):	# add filename and linenum attributes
+    def invoke(self, tex):
+        Command.invoke(self, tex)
+        self.attributes['filename'] = tex.filename
+        self.attributes['linenum'] = tex.lineNumber
+
+class MyBaseVerbatim(Base.verbatim):	# add filename and linenum attributes
+    def invoke(self, tex):
+        self.attributes['filename'] = tex.filename
+        self.attributes['linenum'] = tex.lineNumber
+        return Base.verbatim.invoke(self, tex)
+
+class MyBaseEnvironment(Base.Environment):	# add filename and linenum attributes
+    def invoke(self, tex):
+        self.attributes['filename'] = tex.filename
+        self.attributes['linenum'] = tex.lineNumber
+        return Base.Environment.invoke(self, tex)
+
 class edXcourse(Base.Environment):
     args = '{ number } { display_name } [ attrib_string:str ] self'
 
-class edXchapter(Base.Environment):
+class edXchapter(MyBaseEnvironment):
     args = '{ display_name } [ attrib_string:str ] self'
 
-class edXsection(Base.Environment):
+class edXsection(MyBaseEnvironment):
     # turns into edXsequential
     args = '{ display_name } [ attrib_string:str ] self'
 
@@ -22,7 +40,7 @@ class edXsequential(Base.Environment):
 class edXvertical(Base.Environment):
     args = '{ display_name } [ attrib_string:str ] self'
 
-class edXabox(Base.Command):
+class edXabox(MyBaseCommand):
     args = 'self'
 
 class edXinline(Base.Command):
@@ -44,24 +62,22 @@ class includegraphics(Base.Command):
 class edXcite(Base.Command):	# tooltip citation (appears onmoseover, using <a title="self" href="#"><sup>[ref]</sup></a>)
     args = '[ ref ] self'
 
-class edXinclude(Base.Command):		# include external XML file
+class edXinclude(MyBaseCommand):		# include external XML file
     args = 'self'
 
-class edXincludepy(Base.Command):		# include external python file (puts inside <script>)
+class edXincludepy(MyBaseCommand):		# include external python file (puts inside <script>)
     args = 'self'
 
-class edXdndtex(Base.Command):		# insert external drag-and-drop problem (should point to latex2dnd tex file)
+class edXdndtex(MyBaseCommand):		# insert external drag-and-drop problem (should point to latex2dnd tex file)
     args = '[ attrib_string ] self'
     def invoke(self, tex):
         Command.invoke(self, tex)
-        self.attributes['filename'] = tex.filename
-        self.attributes['linenum'] = tex.lineNumber
-        print "  --> edXdndtex in %s: dndtex=%s, line=%s" % (tex.filename, self.attributes['self'], tex.lineNumber)
+        print "  --> edXdndtex in %s: dndtex=%s, line=%s" % (tex.filename, self.attributes['self'].source, tex.lineNumber)
 
 class edXshowhide(Base.Environment):	# block of text to be hidden by default, but with clickable "show"
     args = ' { id } { description } self'
 
-class edXscript(Base.verbatim):
+class edXscript(MyBaseVerbatim):
     macroName = "edXscript"
     
 class endedXscript(Base.endverbatim):
@@ -88,13 +104,13 @@ class endedXmath(Base.endverbatim):
 class edXxml(Base.Command):
     args = 'self'
 
-class edXproblem(Base.Environment):
+class edXproblem(MyBaseEnvironment):
     args = '{ display_name } { attrib_string } self'
 
-class edXtext(Base.Environment):	# indicates HTML file to be included (ie <html...> in course.xml)
+class edXtext(MyBaseEnvironment):	# indicates HTML file to be included (ie <html...> in course.xml)
     args = '{ display_name } [ attrib_string:str ] self'
 
-class edXsolution(Base.Environment):
+class edXsolution(MyBaseEnvironment):
     args = 'self'
     # note: cannot have \[ immediately after \begin{edXsolution}
 
